
import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import org.gjt.sp.jedit.jEdit;

public class FileListModel extends AbstractListModel {

  static boolean sort;
  static boolean caseSensitive;


  private java.io.File file;
  private FileFilter   filter;
  private boolean      dirs;
  private Vector       list;

  public FileListModel(java.io.File file, FileFilter filter, boolean dirs) {
    this.file   = file;
    this.filter = filter;
    this.dirs = dirs;
    this.list = null;
  }

  public int getSize() {
    return getList().size();
  }

  public Object getElementAt(int index) {
    return getList().elementAt(index);
  }

  public void refresh(FileFilter filter) {
    this.filter = filter;

    if(list == null) list = new Vector();

    synchronized(this) {
      Vector newList = getList(this);

      // first, remove any non-existant children
      Vector removedIndices = new Vector();
      Object o;
      int i, j;
      for(i = 0; i < list.size(); i++) {
        o = list.elementAt(i);
        for(j = newList.size() - 1; j >= 0; j--)
          if(o.equals(newList.elementAt(j))) break;
        if(j < 0) {
          removedIndices.addElement(new Integer(i));
        }
      }
      if(removedIndices.size() > 0) {
        int iStart = -2, iLast = -1, iCurrent;
        for(i = removedIndices.size() - 1; i >= 0; i--) {
          iCurrent = ((Integer)removedIndices.elementAt(i)).intValue();
          list.removeElementAt(iCurrent);
          if(iCurrent != iLast - 1) {
            if(iLast <= iStart) fireIntervalRemoved(this, iLast, iStart);
            iStart = iCurrent;
          }
          iLast = iCurrent;
        }
        if(iLast <= iStart) fireIntervalRemoved(this, iLast, iStart);
      }

      // next, add any new children
      Vector addedIndices = new Vector();
      for(i = 0; i < newList.size(); i++) {
        o = newList.elementAt(i);
        for(j = list.size() - 1; j >= 0; j--)
          if(o.equals(list.elementAt(j))) break;
        if(j < 0) {
          if(sort) {
            for(j = list.size(); j > 0; j--) {
              //if(list.elementAt(j-1) < o) break;
              if(caseSensitive) {
                if(((File)list.elementAt(j-1)).getFile().getName().compareTo(
                   ((File)o).getFile().getName()) < 0) break;
              } else {
                if(((File)list.elementAt(j-1)).getFile().getName()
                   .toLowerCase().compareTo(((File)o).getFile()
                   .getName().toLowerCase()) < 0) break;
              }
            }
          }
          else j = list.size();
          addedIndices.addElement(new Integer(j));
          list.insertElementAt(o, j);
        }
      }
      if(addedIndices.size() > 0) {
        int iStart = -1, iLast = -2, iCurrent;
        for(i = 0; i < addedIndices.size(); i++) {
          iCurrent = ((Integer)addedIndices.elementAt(i)).intValue();
          if(iCurrent != iLast + 1) {
            if(iLast >= iStart) fireIntervalAdded(this, iStart, iLast);
            iStart = iCurrent;
          }
          iLast = iCurrent;
        }
        if(iLast >= iStart) fireIntervalAdded(this, iStart, iLast);
      }
    }
  }

  private Vector getList() {
    if(list == null) {
      synchronized(this) {
        if(list == null) {
          list = getList(this);
        }
      }
    }
    return list;
  }

  private Vector getList(FileListModel model) {
    Vector list = new Vector();

    if(model.file != null && model.file.isDirectory()) {
      String[] fl = model.file.list();
      if(sort) FileTreeNode.sortStrings(fl, caseSensitive);
      for(int i = 0; i < fl.length; i++) {
        java.io.File f = new java.io.File(model.file, fl[i]);
        if(model.dirs == f.isDirectory() &&
           (model.dirs || model.filter == null || model.filter.accept(f)))
          list.addElement(new File(f));
      }
      if(model.dirs) {
        if(jEdit.getBooleanProperty("quickfile.showparent")) {
          String p = model.file.getParent();
          if(p != null)
            list.insertElementAt(new File(new java.io.File(p), File.DOTDOT), 0);
        }
        list.insertElementAt(new File(model.file, File.DOT), 0);
      }
    }

    return list;
  }

  public static class File {

    public static final byte NORMAL = 0;
    public static final byte DOT    = 1;
    public static final byte DOTDOT = 2;

    private final byte mode;
    private final java.io.File file;

    public File(java.io.File file) {
      this.file = file;
      this.mode = NORMAL;
    }

    public File(java.io.File file, byte mode) {
      this.file = file;
      this.mode = mode;
    }

    public java.io.File getFile() {
      return file;
    }

    public boolean equals(Object o) {
      return o instanceof File && file.equals(((File)o).getFile());
    }

    public String toString() {
      switch(mode) {
        case NORMAL: return file.getName();
        case DOT   : return ".";
        case DOTDOT: return "..";
        default    : return null;
      }
    }
  }
}
