/*
 * GlobFileFilter.java - a FileFilter that uses C shell style globbing
 * Copyright (C) 1999 mike dillon
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import gnu.regexp.RE;
import gnu.regexp.REException;
import gnu.regexp.REMatch;
import gnu.regexp.RESyntax;
import java.io.*;
import java.util.*;
import org.gjt.sp.jedit.*;
import javax.swing.filechooser.FileFilter;

/**
 * A FileFilter implementation that filters based on filename globbing.
 *
 * @author mike dillon
 * @author Jason Ginchereau
 */
public class GlobFileFilter extends FileFilter
{
  public GlobFileFilter(String globPattern)
	{
    this.globPattern = globPattern;
		this.rePattern = MiscUtilities.globToRE(globPattern);
    rePattern = "^" + rePattern + "$";  // match whole filename
		int reFlags =
          (jEdit.getBooleanProperty("quickfile.filter.cs")?
           0 : RE.REG_ICASE) | RE.REG_MULTILINE;
		try {
			re = new RE(rePattern, reFlags, RESyntax.RE_SYNTAX_PERL5);
		}
		catch (REException e)
		{
			e.printStackTrace();
		}
  }

  public boolean accept(File f)
	{
		return f.isDirectory()? true : re.getMatch(f.getName()) != null;
  }

  public boolean equals(Object o)
	{
    return globPattern.equals(((GlobFileFilter) o).globPattern);
  }
  
  public String getDescription() {
    // maybe implement something better later
    return toString();
  }

	public String getGlobPattern()
	{
		return globPattern;
	}

  public String getREPattern() {
    return rePattern;
  }

  public String toString()
	{
    return globPattern;
  }

	private String globPattern;
	private String rePattern;
  private RE     re;
}
