
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import org.gjt.sp.jedit.jEdit;

/**
 * Adds tool-tip helpers for obscured tree items.
 *
 * @author Jason Ginchereau
 */
public class HelpfulJTree extends JTree implements MouseListener {

  public HelpfulJTree(TreeNode root) {
    super(root, true);
    if(jEdit.getBooleanProperty("quickfile.tooltiphelpers"))
      ToolTipManager.sharedInstance().registerComponent(this);
    addMouseListener(this);
  }

  public final String getToolTipText(MouseEvent evt) {
    TreePath path = getPathForLocation(evt.getX(), evt.getY());
    if(path != null) {
      Rectangle cellRect = getPathBounds(path);
      if(cellRect != null && !cellRectIsVisible(cellRect)) {
        return path.getLastPathComponent().toString();
      }
    }
    return null;
  }

  public final Point getToolTipLocation(MouseEvent evt) {
    TreePath path = getPathForLocation(evt.getX(), evt.getY());
    if(path != null) {
      Rectangle cellRect = getPathBounds(path);
      if(cellRect != null && !cellRectIsVisible(cellRect)) {
        return new Point(cellRect.x + 17, cellRect.y);
      }
    }
    return null;
  }

  private final boolean cellRectIsVisible(Rectangle cellRect) {
    Rectangle vr = getVisibleRect();
    return vr.contains(cellRect.x + 22, cellRect.y + 2) &&
           vr.contains(cellRect.x + 22 + cellRect.width - 31,
                       cellRect.y + 2 + cellRect.height - 4);
  }

  public void mouseClicked(MouseEvent evt)  { }
  public void mousePressed(MouseEvent evt)  { }
  public void mouseReleased(MouseEvent evt) { }

  public void mouseEntered(MouseEvent evt)  {
    ToolTipManager ttm = ToolTipManager.sharedInstance();
    toolTipInitialDelay = ttm.getInitialDelay();
    toolTipReshowDelay = ttm.getReshowDelay();
    ttm.setInitialDelay(0);
    ttm.setReshowDelay(0);
  }

  public void mouseExited(MouseEvent evt)   {
    ToolTipManager ttm = ToolTipManager.sharedInstance();
    if(toolTipInitialDelay >= 0) ttm.setInitialDelay(toolTipInitialDelay);
    if(toolTipReshowDelay >= 0) ttm.setReshowDelay(toolTipReshowDelay);
  }

  private int toolTipInitialDelay = -1;
  private int toolTipReshowDelay = -1;
}
