/*
 * QuickFileOptionPane.java - Panel in jEdit's Global Options dialog
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;

public class QuickFileOptionPane extends JPanel implements OptionPane {

  private boolean         initialized;

  private JRadioButton    dockRB;
  private JRadioButton    frameRB;
  private JRadioButton    leftRB;
  private JRadioButton    rightRB;

  private JRadioButton[]  viewRBs;

  private JRadioButton    currentRB;
  private JRadioButton    bufferRB;
  private JRadioButton    lastRB;

  private JCheckBox[]     optionCBs;

  static String[] viewStyles = getListProperty("quickfile.views");
  static String[] cbOptions = getListProperty("quickfile.cboptions");

  public void init() {
    if(initialized)
      return;

    initialized = true;

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));

    JPanel p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    JLabel locLabel = new JLabel(jEdit.getProperty("quickfile.location.label"));
    locLabel.setAlignmentX(0);
    p.add(locLabel);
    topPanel.add(p);

    JPanel dockPanel = new JPanel();
    dockPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    dockPanel.add(dockRB = new JRadioButton(
        jEdit.getProperty("quickfile.docked.1.label")));
    dockPanel.add(frameRB = new JRadioButton(
        jEdit.getProperty("quickfile.docked.0.label")));
    topPanel.add(dockPanel);

    ButtonGroup dockBG = new ButtonGroup();
    dockBG.add(dockRB); dockBG.add(frameRB);

    JPanel locPanel = new JPanel();
    locPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    locPanel.add(leftRB = new JRadioButton(
        jEdit.getProperty("quickfile.side.0.label")));
    locPanel.add(rightRB = new JRadioButton(
        jEdit.getProperty("quickfile.side.1.label")));
    topPanel.add(locPanel);

    ButtonGroup locationBG = new ButtonGroup();
    locationBG.add(leftRB); locationBG.add(rightRB);

    Dimension space = new Dimension(0, 10);
    topPanel.add(new Box.Filler(space, space, space));

    JLabel viewLabel = new JLabel(
        jEdit.getProperty("quickfile.view.default.label"));
    viewLabel.setAlignmentX(0);
    p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    p.add(viewLabel);
    topPanel.add(p);
    JPanel viewPanel = new JPanel();
    viewPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    viewRBs = new JRadioButton[viewStyles.length];
    ButtonGroup viewBG = new ButtonGroup();
    for(int i = 0; i < viewStyles.length; i++) {
      viewRBs[i] = new JRadioButton(
          jEdit.getProperty("quickfile.view." + viewStyles[i] + ".label"));
      viewPanel.add(viewRBs[i]);
      viewBG.add(viewRBs[i]);
    }
    topPanel.add(viewPanel);

    topPanel.add(new Box.Filler(space, space, space));

    JLabel startLabel = new JLabel(jEdit.getProperty("quickfile.start.label"));
    startLabel.setAlignmentX(0);
    p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    p.add(startLabel);
    topPanel.add(p);
    currentRB =
        new JRadioButton(jEdit.getProperty("quickfile.start.current.label"));
    currentRB.setAlignmentX(0);
    p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    p.add(currentRB);
    topPanel.add(p);
    bufferRB =
        new JRadioButton(jEdit.getProperty("quickfile.start.parent.label"));
    bufferRB.setAlignmentX(0);
    p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    p.add(bufferRB);
    topPanel.add(p);
    lastRB =
        new JRadioButton(jEdit.getProperty("quickfile.start.last.label"));
    lastRB.setAlignmentX(0);
    p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    p.add(lastRB);
    topPanel.add(p);

    ButtonGroup startBG = new ButtonGroup();
    startBG.add(currentRB); startBG.add(bufferRB); startBG.add(lastRB);

    topPanel.add(new Box.Filler(space, space, space));

    JPanel cbPanel = new JPanel();
    cbPanel.setLayout(new GridLayout(cbOptions.length, 1, 0, 0));

    optionCBs = new JCheckBox[cbOptions.length];
    for(int i = 0; i < optionCBs.length; i++) {
      optionCBs[i] = new JCheckBox(
          jEdit.getProperty("quickfile." + cbOptions[i] + ".label"));
      cbPanel.add(optionCBs[i]);
    }

    JScrollPane cbScroller = new JScrollPane(cbPanel,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER) {
      public Dimension getPreferredSize() {
        Dimension ps = super.getPreferredSize();
        if(ps.height > 100) ps.height = 100;
        return ps;
      }
    };
    cbScroller.setBorder(null);
    add(cbScroller);

    setLayout(new BorderLayout());
    add(BorderLayout.NORTH, topPanel);
    add(BorderLayout.CENTER, cbScroller);

    load();
  }

  public String getName() {
    return "quickfile";
  }

  public Component getComponent() {
    return this;
  }

  private void load() {
    if(jEdit.getBooleanProperty("quickfile.docked")) dockRB.setSelected(true);
    else frameRB.setSelected(true);
    if(jEdit.getBooleanProperty("quickfile.side")) rightRB.setSelected(true);
    else leftRB.setSelected(true);

    String start = jEdit.getProperty("quickfile.start");
    if(start.equals("current")) currentRB.setSelected(true);
    else if(start.equals("last")) lastRB.setSelected(true);
    else if(start.equals("buffer")) bufferRB.setSelected(true);

    String view = jEdit.getProperty("quickfile.view.default");
    for(int i = 0; i < viewStyles.length; i++)
      if(view.equals(viewStyles[i])) viewRBs[i].setSelected(true);

    for(int i = 0; i < optionCBs.length; i++) {
      optionCBs[i].setSelected(jEdit.getBooleanProperty("quickfile." + cbOptions[i]));
    }
  }

  /**
   * Called when the options dialog's `OK' button is pressed.
   * This should save any properties saved in this option pane.
   */
  public void save() {
    if(!initialized)
      return;

    jEdit.setBooleanProperty("quickfile.docked",dockRB.isSelected());
    jEdit.setBooleanProperty("quickfile.side",rightRB.isSelected());

    String start = "current";
    if(bufferRB.isSelected()) start = "buffer";
    else if(lastRB.isSelected()) start = "last";
    jEdit.setProperty("quickfile.start", start);

    String view = jEdit.getProperty("quickfile.view.default");
    for(int i = 0; i < viewRBs.length; i++)
      if(viewRBs[i].isSelected()) {
        view = viewStyles[i];
        break;
      }
    jEdit.setProperty("quickfile.view.default", view);

    for(int i = 0; i < optionCBs.length; i++) {
      String optionName = "quickfile." + cbOptions[i];
      jEdit.setBooleanProperty(optionName, optionCBs[i].isSelected());
    }
  }

  public static String[] getListProperty(String propName) {
    String p = jEdit.getProperty(propName);
    if(p == null) {
      return new String[0];
    }
    else {
      StringTokenizer st = new StringTokenizer(p, ", \t\r\n");
      String[] array = new String[st.countTokens()];
      for(int i = 0; i < array.length; i++)
        array[i] = st.nextToken();
      return array;
    }
  }
  
  public static int getIntegerProperty(String prop, int defaultVal) {
    try {
      return Integer.parseInt(jEdit.getProperty(prop));
    }
    catch(Exception ex) {
      return defaultVal;
    }
  }
}
