/*
 * QuickFileView.java - GUI class of the QuickFile plugin for jEdit
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import org.gjt.sp.jedit.*;

/**
 *
 * @author Jason Ginchereau
 */
public abstract class QuickFileView extends JPanel {

  protected File       dir;
  protected FileFilter filter;

  private Vector listeners;

  public QuickFileView(File dir) {
    this.dir = dir;
    this.filter = null;

    this.listeners = new Vector();

    setOpaque(false);
  }

  public void addQuickFileViewListener(QuickFileViewListener l) {
    listeners.addElement(l);
  }

  public void removeQuickFileViewListener(QuickFileViewListener l) {
    listeners.removeElement(l);
  }

  protected void fireFileSelected(File file) {
    for(Enumeration e = listeners.elements(); e.hasMoreElements(); )
      ((QuickFileViewListener)e.nextElement()).fileSelected(this, file);
  }

  protected void fireFileActivated(File file) {
    for(Enumeration e = listeners.elements(); e.hasMoreElements(); )
      ((QuickFileViewListener)e.nextElement()).fileActivated(this, file);
  }

  public File getRootDirectory() {
    return dir;
  }

  public abstract File getSelectedDirectory();

  public abstract File getSelectedFile();

  public FileFilter getFilter() {
    return filter;
  }

  public void setFilter(FileFilter filter) {
    this.filter = filter;
  }
  
  public Dimension getMinimumSize() {
    return new Dimension(0, super.getMinimumSize().height);
  }

  /**
   * Refreshes this view by checking for newly created or deleted files.
   * Also re-applies the filter.
   */
  public abstract void refresh();

  protected void showFilePopup(File file, Component c, Point p) {
    QuickFilePopupMenu popup = new QuickFilePopupMenu(file, this);
    popup.show(c, p.x, p.y);
  }
}
